@@ -17,7 +17,6 @@ from functools import wraps
 import numpy as np
 import pandas as pd
 import pysam
-import xopen
 
 import celescope.tools
 from celescope.tools.__init__ import __PATTERN_DICT__
@@ -236,92 +235,14 @@ def get_id_name_dict(gtf_file):
     return id_name
 
 
-@add_log
-def process_read(
-        read2_file, pattern_dict, barcode_dict, linker_dict,
-        barcode_length, linker_length):
-
-    # if valid, return (True)
-    metrics = defaultdict(int)
-    res_dict = genDict(dim=3)
-    read2 = xopen.xopen(read2_file, "rt")
-    while True:
-        line1 = read2.readline()
-        line2 = read2.readline()
-        read2.readline()
-        line4 = read2.readline()
-        if not line4:
-            break
-        metrics['Total Reads'] += 1
-        attr = str(line1).strip("@").split("_")
-        barcode = str(attr[0])
-        umi = str(attr[1])
-        seq = line2.strip()
-        if linker_length != 0:
-            seq_linker = seq_ranges(seq, pattern_dict['L'])
-            if len(seq_linker) < linker_length:
-                metrics['Reads Unmapped too Short'] += 1
-                continue
-        if barcode_dict:
-            seq_barcode = seq_ranges(seq, pattern_dict['C'])
-            if barcode_length != len(seq_barcode):
-                miss_length = barcode_length - len(seq_barcode)
-                if miss_length > 2:
-                    metrics['Reads Unmapped too Short'] += 1
-                    continue
-                seq_barcode = seq_barcode + "A" * miss_length
-
-        # check linker
-        if linker_length != 0:
-            valid_linker = False
-            for linker_name in linker_dict:
-                if hamming_correct(linker_dict[linker_name], seq_linker):
-                    valid_linker = True
-                    break
-        else:
-            valid_linker = True
-
-        if not valid_linker:
-            metrics['Reads Unmapped Invalid Linker'] += 1
-            continue
-
-        # check barcode
-        valid_barcode = False
-        for barcode_name in barcode_dict:
-            if hamming_correct(barcode_dict[barcode_name], seq_barcode):
-                res_dict[barcode][barcode_name][umi] += 1
-                valid_barcode = True
-                break
-
-        if not valid_barcode:
-            metrics['Reads Unmapped Invalid Barcode'] += 1
-            continue
-
-        # mapped
-        metrics['Reads Mapped'] += 1
-        if metrics['Reads Mapped'] % 1000000 == 0:
-            process_read.logger.info(str(metrics['Reads Mapped']) + " reads done.")
-
-    return res_dict, metrics
-
-
-def seq_ranges_exception(seq, pattern_dict):
-    # get subseq with intervals in arr and concatenate
-    length = len(seq)
-    for x in pattern_dict:
-        if length < x[1]:
-            raise Exception(f'invalid seq range {x[0]}:{x[1]} in read')
-    return ''.join([seq[x[0]:x[1]]for x in pattern_dict])
-
-
-def seq_ranges(seq, pattern_dict):
-    # get subseq with intervals in arr and concatenate
-    return ''.join([seq[x[0]:x[1]]for x in pattern_dict])
-
-
 def read_one_col(file):
+    """
+    Read file with one column. Strip each line.
+    Returns col_list, line number
+    """
     df = pd.read_csv(file, header=None)
     col1 = list(df.iloc[:, 0])
+    col1 = [item.strip() for item in col1]
     num = len(col1)
     return col1, num
 
